@@ -143,7 +143,7 @@ typedef struct nv_ioctl_rm_api_old_version
 
 typedef enum {
     NV_CPU_TYPE_UNKNOWN = 0,
-    NV_CPU_TYPE_ARM_A15
+    NV_CPU_TYPE_ARM_A9
 } nv_cpu_type_t;
 
 /* alloc event */
@@ -294,6 +294,17 @@ typedef struct nv_kern_mapping_s
     struct nv_kern_mapping_s *next;
 } nv_kern_mapping_t;
 
+typedef struct nv_mmap_context_s
+{
+    NvP64 addr;
+    NvU64 size;
+    NvU32 process_id;
+    NvU32 thread_id;
+    void *file;
+    void *os_priv;
+    struct nv_mmap_context_s *next;
+} nv_mmap_context_t;
+
 /*
  * per device state
  */
@@ -331,6 +342,8 @@ typedef struct
 
     nv_kern_mapping_t *kern_mappings;
 
+    nv_mmap_context_t *mmap_contexts;
+
 } nv_state_t;
 
 // Forward define the gpu ops structures
@@ -360,7 +373,7 @@ typedef struct
 #define NV_FLAG_MAP_REGS_EARLY         0x0008
 #define NV_FLAG_USE_BAR0_CFG           0x0010
 #define NV_FLAG_USES_MSI               0x0020
-#define NV_FLAG_S4_AVAILABLE           0x0040
+// EMPTY SLOT                          0x0040
 #define NV_FLAG_PASSTHRU               0x0080
 #define NV_FLAG_GVI_IN_SUSPEND         0x0100
 #define NV_FLAG_GVI                    0x0200
@@ -455,6 +468,14 @@ typedef struct
     ((offset) >= (nv)->ud.cpu_address) &&                                       \
     (((offset) + ((length)-1)) <= (nv)->ud.cpu_address + ((nv)->ud.size-1)))
 
+#define IS_IMEM_OFFSET(nv, offset, length)                                      \
+    (((nv)->bars[NV_GPU_BAR_INDEX_IMEM].cpu_address != 0) &&                    \
+     ((nv)->bars[NV_GPU_BAR_INDEX_IMEM].size != 0) &&                           \
+     ((offset) >= (nv)->bars[NV_GPU_BAR_INDEX_IMEM].cpu_address) &&             \
+     (((offset) + ((length) - 1)) <=                                            \
+        (nv)->bars[NV_GPU_BAR_INDEX_IMEM].cpu_address +                         \
+            ((nv)->bars[NV_GPU_BAR_INDEX_IMEM].size - 1)))
+
 /* device name length; must be atleast 8 */
 
 #define NV_DEVICE_NAME_LENGTH 40
@@ -532,6 +553,7 @@ void*      NV_API_CALL  nv_alloc_kernel_mapping  (nv_state_t *, void *, NvU64, N
 RM_STATUS  NV_API_CALL  nv_free_kernel_mapping   (nv_state_t *, void *, void *, void *);
 RM_STATUS  NV_API_CALL  nv_alloc_user_mapping    (nv_state_t *, void *, NvU64, NvU32, NvU64, NvU32, NvU64 *, void **);
 RM_STATUS  NV_API_CALL  nv_free_user_mapping     (nv_state_t *, void *, NvU64, void *);
+RM_STATUS  NV_API_CALL  nv_create_user_mapping_context (nv_state_t *, NvU64, NvU64, void **);
 
 NvU64  NV_API_CALL  nv_get_kern_phys_address     (NvU64);
 NvU64  NV_API_CALL  nv_get_user_phys_address     (NvU64);
@@ -597,6 +619,8 @@ BOOL       NV_API_CALL  rm_free_private_state    (nv_stack_t *, nv_state_t *);
 BOOL       NV_API_CALL  rm_init_adapter          (nv_stack_t *, nv_state_t *);
 BOOL       NV_API_CALL  rm_disable_adapter       (nv_stack_t *, nv_state_t *);
 BOOL       NV_API_CALL  rm_shutdown_adapter      (nv_stack_t *, nv_state_t *);
+RM_STATUS  NV_API_CALL  rm_extract_mmap_context  (nv_stack_t *, nv_state_t *, NvP64, NvU64, void **);
+void       NV_API_CALL  rm_purge_mmap_contexts   (nv_stack_t *, nv_state_t *, void *);
 RM_STATUS  NV_API_CALL  rm_validate_mmap_request (nv_stack_t *, nv_state_t *, void *, NvU64, NvU64, NvU32 *, void **, NvU64 *);
 RM_STATUS  NV_API_CALL  rm_acquire_api_lock      (nv_stack_t *);
 RM_STATUS  NV_API_CALL  rm_release_api_lock      (nv_stack_t *);
