@@ -853,6 +853,17 @@ RM_STATUS NV_API_CALL nv_free_kernel_mapping(
     return RM_OK;
 }
 
+RM_STATUS NV_API_CALL nv_create_user_mapping_context(
+    nv_state_t *nv,
+    NvU64 address,
+    NvU64 size,
+    void **ppPrivate
+)
+{
+    *ppPrivate = NULL;
+    return RM_OK;
+}
+
 RM_STATUS NV_API_CALL nv_alloc_user_mapping(
     nv_state_t *nv,
     void       *pAllocPrivate,
